// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.key.polytomous;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.key.KeyEditor;
import eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider;

/**
 * @author n.hoffmann
 * @created Mar 31, 2011
 * @version 1.0
 */
public class PolytomousKeyListEditor extends EditorPart implements
		IConversationEnabled, IDirtyMarkableSelectionProvider,
		IPolytomousKeyEditorPage {

	private class LinkListener extends MouseAdapter {
		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events
		 * .MouseEvent)
		 */
		@Override
		public void mouseUp(MouseEvent event) {

		    if(event.button == 1 && event.count == 2) {
		        Table table = (Table) event.widget;
		        // Determine where the mouse was clicked
		        Point point = new Point(event.x, event.y);

		        int selectedColumn = getSelectedColumn(table, point);
		        PolytomousKeyNode node = (PolytomousKeyNode) getTableItem(
                        table, point).getData();
                
		        if (selectedColumn == 4) {		
		            PolytomousKeyNode linkData = getItemLinkData(node);
		            if (linkData != null) {
		                viewer.setSelection(new StructuredSelection(linkData), true);
		            } 
		        }
		        if (selectedColumn == 5) {
		            Taxon taxon = getItemTaxon(node);
		            if (taxon != null) {
		                try {
		                    EditorUtil.openTaxonBase((taxon).getUuid());
		                } catch (PartInitException e) {
		                    EditorUtil.error(getClass(), e);
		                }
		            }
		        }
		    }
		}

		private int getSelectedColumn(Table table, Point point) {
			TableItem item = getTableItem(table, point);
			if (item != null) {
				for (int i = 0, n = table.getColumnCount(); i < n; i++) {
					Rectangle rect = item.getBounds(i);
					if (rect.contains(point)) {
						// This is the selected column
						return i;
					}
				}
			}
			return -1;
		}

		private TableItem getTableItem(Table table, Point point) {
			return table.getItem(point);
		}

		/**
		 * @return
		 */
		private PolytomousKeyNode getItemLinkData(PolytomousKeyNode node) {
			return node.getChildren().isEmpty() ? null : node
					.getChildAt(0);
		}
		
	      /**
         * @return
         */
        private Taxon getItemTaxon(PolytomousKeyNode node) {
            return node.getTaxon();
        }
	}

	public static final String ID = "eu.etaxonomy.taxeditor.editor.key.polytomous.list";

	private TableViewer viewer;

	private final KeyEditor editor;

	public PolytomousKeyListEditor(KeyEditor editor) {
		this.editor = editor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update
	 * (eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap arg0) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder
	 * ()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return ((PolytomousKeyEditorInput) getEditorInput())
				.getConversationHolder();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite,
	 * org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {

		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;
		parent.setLayout(fillLayout);

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		getSite().setSelectionProvider(viewer);

		createColumns(viewer);
		viewer.getControl().addMouseListener(new LinkListener());
		viewer.setContentProvider(new PolytomousKeyListContentProvider());
		viewer.setLabelProvider(new PolytomousKeyListLabelProvider());

		createMenu();

		PolytomousKey key = ((PolytomousKeyEditorInput) getEditorInput())
				.getKey();

		setPartName(key.getTitleCache());

		viewer.setInput(key);
	}

	public int getTableItemCount() {
	    if (viewer != null && viewer.getTable() != null) {
	        return viewer.getTable().getItemCount();
	    }
	    
	    return 0;
	}
	
	public PolytomousKey getViewerInputKey() {
	    return (PolytomousKey) viewer.getInput();
	}
	private void createMenu() {
		// register context menu
		MenuManager menuManager = new MenuManager();
		menuManager
				.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(ID, menuManager, viewer);

		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);
		control.setMenu(menu);
	}

	// This will create the columns for the table
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = { "Node", "Question", "Edge",  "Statement", "Link", "Taxon" };
		int[] bounds = { 50, 200, 50, 200, 100, 200 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(false);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider#changed(
	 * java.lang.Object)
	 */
	@Override
	public void changed(Object element) {
	    if(element != null) {
	        viewer.update(element, null);
	    }

		if (element instanceof PolytomousKeyNode) {
			List<PolytomousKeyNode> children = ((PolytomousKeyNode) element)
					.getParent().getChildren();
			for (PolytomousKeyNode child : children) {
				viewer.update(child, null);
			}
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#postOperation(
	 * eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		viewer.refresh();

		editor.changed(objectAffectedByOperation);

		if (objectAffectedByOperation != null) {
			viewer.setSelection(new StructuredSelection(
					objectAffectedByOperation), true);
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#onComplete()
	 */
	@Override
	public boolean onComplete() {
		return true;
	}

}
