/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>DeleteSynonymOperation class.</p>
 *
 * @author p.ciardelli
 * @created 14.01.2009
 * @version 1.0
 */
public class DeleteSynonymOperation extends AbstractPostOperation {
	
	private Synonym synonym;
	private Set<SynonymRelationshipType> synonymTypes;

	/**
	 * <p>Constructor for DeleteSynonymOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param synonym a {@link eu.etaxonomy.cdm.model.taxon.Synonym} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public DeleteSynonymOperation(String label, IUndoContext undoContext,
			Taxon taxon, Synonym synonym, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.synonym = synonym;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Store synonymRelationshipType for later undo operations
				synonymTypes = synonym.getRelationType(taxon);
				monitor.worked(20);
				
				// Remove synonym from taxon
				
				CdmApplicationController controller;
				
				controller = (CdmApplicationController) CdmStore.getCurrentApplicationConfiguration();
				
				ITaxonService service = controller.getTaxonService();
				if (synonym.getId() == 0){
					taxon.removeSynonym(synonym);
					
				} else {
					service.deleteSynonym(synonym, null);
				}
			//	taxon.removeSynonym(synonym);
//				CdmStore.getTaxonService().deleteSynonymRelationships(synonym);
//				CdmStore.getTaxonService().delete(synonym);
				
				monitor.worked(40);

				// Redraw editor if exists
				
				return postExecute(taxon);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {		
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Add synonym to taxon
		for (SynonymRelationshipType synonymType : synonymTypes){
			taxon.addSynonym(synonym, synonymType);
		}
		
		// Redraw editor if exists
		return postExecute(synonym);
	}
}
