package eu.etaxonomy.taxeditor.editor.name.operation;

import java.util.Iterator;
import java.util.Set;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.api.service.exception.DataChangeNoRollbackException;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

public class DeleteTaxonOperation extends DeleteTaxonBaseOperation{
	//private Taxon taxon;
	private Classification classification;
	
	
	
	
	public DeleteTaxonOperation(String label, IUndoContext undoContext,
			Taxon taxon, Classification classification, IWorkbenchPage activePage, IPostOperationEnabled postOperationEnabled, IConversationEnabled conversationEnabled) {
		super(label, undoContext, taxon, activePage, postOperationEnabled, conversationEnabled);
		this.taxon = taxon;
		
		Set<TaxonNode> nodes = taxon.getTaxonNodes();
		if (nodes.size() == 1 && classification == null){
			this.taxonNode = nodes.iterator().next();
		} else if (classification != null){
			Iterator<TaxonNode> iterator = nodes.iterator();
			while (iterator.hasNext()){
				TaxonNode node = iterator.next();
				if (node.getClassification().equals(classification)){
					this.taxonNode = node;
				}
			}
		} else{
			//TODO
		}
		this.classification = classification;
		
	}

	

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
				
				monitor.worked(20);
				bind();
				
				
				CdmApplicationController controller;
				
				controller = (CdmApplicationController) CdmStore.getCurrentApplicationConfiguration();
				
				ITaxonService service = controller.getTaxonService();
				try {
					
					service.deleteTaxon(taxon, new TaxonDeletionConfigurator(), classification);
					
				} catch (DataChangeNoRollbackException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					
				}
				
				//closeObsoleteEditor(taxon);
				monitor.worked(40);
				
				
				return postExecute(null);
	}

	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}
	
	
}
