/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.concept.operation;

import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * Deletes given relations between two taxa.
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 29.01.2009
 * @version 1.0
 */
public class DeleteConceptRelationOperation extends AbstractPostOperation {
	
	private Set<TaxonRelationship> taxonRelationships;	


	/**
	 * @param name
	 * @param undoContext
	 * @param relation
	 * @param postOperationEnabled
	 */
	public DeleteConceptRelationOperation(String label,
			IUndoContext undoContext, Taxon taxon, Set<TaxonRelationship> relations,
			IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		
		taxonRelationships = relations;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		monitor.worked(20);
		
		// Remove relation from taxon
		for(TaxonRelationship relationship : taxonRelationships){
			taxon.removeTaxonRelation(relationship);
			monitor.worked(10);
		}
		monitor.worked(10);

		return postExecute(taxon);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		for(TaxonRelationship relationship : taxonRelationships){
			taxon.addTaxonRelation(relationship);
			monitor.worked(10);
		}
		
		return postExecute(taxon);
	}
}
