/**
* Copyright (C) 2011 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.uses.operation;

import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.UsageTermCollection;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.CreateTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * CreateTaxonUseOperation Class
 * @author a.theys	
 * @created mar 13, 2012
 * @version 1.0
 */
public class CreateTaxonUseOperation extends CreateTaxonDescriptionOperation {
	private TaxonDescription description;
	private Marker marker;

	/**
	 * <p>Constructor for CreateTaxonUseOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 */
	public CreateTaxonUseOperation(String label, IUndoContext undoContext,
			Taxon taxon, IPostOperationEnabled postOperationEnabled) {
		this(label, undoContext, taxon, postOperationEnabled, false);
	}

	/**
	 * <p>Constructor for CreateTaxonUseOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxon a {@link eu.etaxonomy.cdm.model.taxon.Taxon} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param isImageGallery a boolean.
	 */
	public CreateTaxonUseOperation(String label, IUndoContext undoContext,
			Taxon taxon, IPostOperationEnabled postOperationEnabled, boolean isImageGallery) {
		super(label, undoContext, taxon, postOperationEnabled);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		description = TaxonDescription.NewInstance(taxon);
		monitor.worked(20);
		MarkerType useMarkerType = (MarkerType) CdmStore.getService(ITermService.class).find(UsageTermCollection.uuidUseMarkerType);
		marker = Marker.NewInstance(useMarkerType, true);
		description.addMarker(marker);
		monitor.worked(40);

		return postExecute(description);
	}
}

