// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.store.operations.AbstractTaxeditorOperationTestBase;

/**
 * @author n.hoffmann
 * @created 08.04.2009
 * @version 1.0
 */
public class DeleteConceptRelationOperationTest extends AbstractTaxeditorOperationTestBase{
	private static final Logger logger = Logger
			.getLogger(DeleteConceptRelationOperationTest.class);

	private static TaxonRelationshipType relationshipType;

	private static Taxon relatedTaxon;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		
		taxon = Taxon.NewInstance(null, null);
		relatedTaxon = Taxon.NewInstance(null, null);
		relationshipType = TaxonRelationshipType.CONTRADICTION();
		
		taxon.addTaxonRelation(relatedTaxon, relationshipType, null, null);
		
		Assert.assertTrue(taxon.getTaxonRelations().size() > 0);
		
//		operation = new DeleteConceptRelationOperation("", undoContext, taxon, relatedTaxon, postOperation);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.view.concept.operation.DeleteConceptRelationOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecuteIProgressMonitorIAdaptable() throws ExecutionException {
		operation.execute(monitor, info);
		
		Assert.assertTrue(taxon.getTaxonRelations().size() == 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.view.concept.operation.DeleteConceptRelationOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.undo(monitor, info);
		
		Assert.assertTrue(taxon.getTaxonRelations().size() > 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.view.concept.operation.DeleteConceptRelationOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.redo(monitor, info);
		
		Assert.assertTrue(taxon.getTaxonRelations().size() == 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operation.AbstractPostOperation#postExecute(ICdmBase)}.
	 */
	@Test
	public void testPostExecute() {
		// TODO there is not post operation functionality for this class
		// at the moment. Implement test when there is.
		logger.warn("No post operation functionality for this class");
	}
}
